package com.gic.enterprise.service.outer.impl;

import java.util.*;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dto.AppletCardDTO;
import com.gic.enterprise.entity.TabAppletDecorationConfig;
import com.gic.member.config.api.dto.MemberCardRelDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.AppletDataAssetAreaDTO;
import com.gic.enterprise.dto.AppletDecorationConfigDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.AppletDecorationConfigApiService;
import com.gic.enterprise.service.AppletDecorationConfigService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AppletCardRelationDTO;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;

@Service("appletDecorationConfigApiService")
public class AppletDecorationConfigApiServiceImpl implements AppletDecorationConfigApiService {
    private static final Logger LOGGER = LogManager.getLogger(AppletDecorationConfigApiServiceImpl.class);
    @Autowired
    private AppletDecorationConfigService appletDecorationConfigService;
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @Override
    public ServiceResponse<Integer> save(AppletDecorationConfigDTO dto) {

        ServiceResponse valueValidResult = valueValid(dto);
        if (!valueValidResult.isSuccess()) {
            return ServiceResponse.failure(valueValidResult.getCode(), valueValidResult.getMessage());
        }

        //
        TabAppletDecorationConfig record = appletDecorationConfigService.getByAppId(dto.getAppId(),
                dto.getEnterpriseId());
        if (record != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不得重复配置");
        }

        if (dto.getGpsDecisionCard() == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        }

        //默认选中和主卡关联的店铺，不可编辑
        ServiceResponse<ShopDTO> shopResult = getMainCardStore(dto.getAppId(), dto.getEnterpriseId());
        if (!shopResult.isSuccess()) {
            return ServiceResponse.failure(shopResult.getCode(), shopResult.getMessage());
        }
        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();
        ShopDTO shopDTO = shopResult.getResult();
        appletDataAssetArea.setMallOrder(shopDTO.getId().toString());
        appletDataAssetArea.setMallOrderName(shopDTO.getName());

        //数据资产设置
        dto.setDataAssetArea(JSON.toJSONString(appletDataAssetArea));
        return ServiceResponse.success(appletDecorationConfigService.save(dto));
    }

    @Override
    public ServiceResponse<String> edit(AppletDecorationConfigDTO dto) {
        TabAppletDecorationConfig record = appletDecorationConfigService.getById(dto.getConfigId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "装修配置ID错误，查无数据");
        }
        dto.setAppId(record.getAppId());
        dto.setEnterpriseId(record.getEnterpriseId());

        ServiceResponse valueValidResult = valueValid(dto);
        if (!valueValidResult.isSuccess()) {
            return ServiceResponse.failure(valueValidResult.getCode(), valueValidResult.getMessage());
        }

        if (dto.getGpsDecisionCard() == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        }
        //默认选中和主卡关联的店铺，不可编辑 //实时调用
        ServiceResponse<ShopDTO> shopResult = getMainCardStore(record.getAppId(), record.getEnterpriseId());
        if (!shopResult.isSuccess()) {
            return ServiceResponse.failure(shopResult.getCode(), shopResult.getMessage());
        }

        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();

        ShopDTO shopDTO = shopResult.getResult();
        appletDataAssetArea.setMallOrder(shopDTO.getId().toString());
        appletDataAssetArea.setMallOrderName(shopDTO.getName());

        //数据资产设置
        dto.setDataAssetArea(JSON.toJSONString(appletDataAssetArea));
        appletDecorationConfigService.edit(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<AppletDecorationConfigDTO> getDetailByAppId(String appId, Integer enterpriseId) {
        TabAppletDecorationConfig record = appletDecorationConfigService.getByAppId(appId, enterpriseId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "尚未配置信息");
        }
        AppletDecorationConfigDTO dto = EntityUtil.changeEntityNew(AppletDecorationConfigDTO.class, record);
        AppletDataAssetAreaDTO old = JSON.parseObject(record.getDataAssetArea(), AppletDataAssetAreaDTO.class);
        dto.setAppletDataAssetArea(old);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<Object> listAppletCard(String appId, Integer enterpriseId) {
        ServiceResponse<List<AppletCardDTO>> result = getMemberCardByAppId(appId, enterpriseId);
        if (result.isSuccess()) {
            return ServiceResponse.success(result.getResult());
        }
        return ServiceResponse.failure(result.getCode(), result.getMessage());
    }

    private ServiceResponse valueValid(AppletDecorationConfigDTO dto) {
        String appId = dto.getAppId();
        Integer enterpriseId = dto.getEnterpriseId();
        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();
        if (StringUtils.isBlank(appId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序ID不能为空");
        }
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (appletDataAssetArea == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产配置不能为空");
        }
        //校验参数数值是否非法
        ServiceResponse<List<AppletCardDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
        if (!appletCardResult.isSuccess()) {
            return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
        }
        List<AppletCardDTO> appletCardList = appletCardResult.getResult();
        Map<String, Boolean> appletCardMap = new HashMap<>(16);
        Map<String, List<AppletCardDTO>> appletMainCardMap = new HashMap<>(16);
        for (AppletCardDTO cardDTO : appletCardList) {
            appletCardMap.put(cardDTO.getCardConfigId().toString(), cardDTO.getIsSub());
            if (!cardDTO.getIsSub()) {
                appletMainCardMap.put(cardDTO.getCardConfigId().toString(), cardDTO.getSubList());
            }
        }

        //需要校验的数值集合
        Map<String, String> needValidMap = new HashMap<>(16);
        needValidMap.put("会员中心页面卡范围", dto.getMemberCardArea());
        needValidMap.put("开卡认证关联", dto.getOpenCardAuth());
        //数据资产
        //总积分
        String totalIntegral = appletDataAssetArea.getIntegral();
        needValidMap.put("数据资产-总积分", totalIntegral);
        needValidMap.put("数据资产-卡券包", appletDataAssetArea.getCoupon());
        needValidMap.put("数据资产-零售订", appletDataAssetArea.getOrder());

        ServiceResponse valueValid;
        for (Map.Entry<String, String> entry : needValidMap.entrySet()) {
            valueValid = validParamValue(entry.getKey(), entry.getValue(), appletCardMap);
            if (!valueValid.isSuccess()) {
                return ServiceResponse.failure(valueValid.getCode(), valueValid.getMessage());
            }
        }

        if (totalIntegral.split(",").length > 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-总积分只能选择一个");
        }
        //如果可用积分是空的，则默认和总积分一样
        String availableIntegral = appletDataAssetArea.getAvailableIntegral();
        if (StringUtils.isBlank(availableIntegral)) {
            appletDataAssetArea.setAvailableIntegral(totalIntegral);
            //签到
            appletDataAssetArea.setSignIn(totalIntegral);
            //会员成就
            appletDataAssetArea.setMemberAchievement(totalIntegral);
            //会员任务
            appletDataAssetArea.setMemberTask(totalIntegral);
        } else {
            Boolean isSub = appletCardMap.get(totalIntegral);
            if (isSub) {
                //如果总积分选择的是子卡，则下面的只能是子卡
                appletDataAssetArea.setAvailableIntegral(totalIntegral);
                //签到
                appletDataAssetArea.setSignIn(totalIntegral);
                //会员成就
                appletDataAssetArea.setMemberAchievement(totalIntegral);
                //会员任务
                appletDataAssetArea.setMemberTask(totalIntegral);
            } else {
                //如果总积分是主卡，则可用积分可以是主卡，也可以下面子卡
                Boolean isSubOfAvailableIntegral = appletCardMap.get(availableIntegral);
                List<AppletCardDTO> subListTemp = appletMainCardMap.get(totalIntegral);
                Set<String> subSet = new HashSet<>();
                if (CollectionUtils.isNotEmpty(subListTemp)) {
                    for (AppletCardDTO subSetTemp : subListTemp) {
                        subSet.add(subSetTemp.getCardConfigId().toString());
                    }
                }
                if (isSubOfAvailableIntegral) {
                    if (!subSet.contains(availableIntegral)) {
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-可用积分不在总积分会员卡及其子卡");
                    }
                } else {
                    //
                    if (!totalIntegral.equals(availableIntegral)) {
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-可用积分不在总积分会员卡及其子卡");
                    }
                    String signIn = appletDataAssetArea.getSignIn();
                    if (StringUtils.isBlank(signIn)) {
                        appletDataAssetArea.setSignIn(availableIntegral);
                    }
                    if (!subSet.contains(signIn)) {
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-签到不在可用积分会员卡及其子卡");
                    }
                    String memberTask = appletDataAssetArea.getMemberTask();
                    if (StringUtils.isBlank(memberTask)) {
                        appletDataAssetArea.setMemberTask(availableIntegral);
                    }
                    if (!subSet.contains(memberTask)) {
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-会员任务不在可用积分会员卡及其子卡");
                    }
                    String memberAchievement = appletDataAssetArea.getMemberAchievement();
                    if (StringUtils.isBlank(memberAchievement)) {
                        appletDataAssetArea.setMemberAchievement(availableIntegral);
                    }
                    if (!subSet.contains(memberAchievement)) {
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-会员成就不在可用积分会员卡及其子卡");
                    }

                }

            }
        }

        Integer gpsDecisionCard = dto.getGpsDecisionCard();
        if (gpsDecisionCard == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        } else {
            if (gpsDecisionCard != 1 && gpsDecisionCard != 0) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "GPS智能判定卡数值非法");
            }
        }
        dto.setAppletDataAssetArea(appletDataAssetArea);
        return ServiceResponse.success();
    }


    private ServiceResponse validParamValue(String key, String value, Map<String, Boolean> appletCardMap) {
        if (StringUtils.isBlank(value)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), key + "不能为空");
        }
        String[] arr = value.split(",");
        for (String str : arr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            if (!appletCardMap.containsKey(str)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), key + "数值非法");
            }
        }
        return ServiceResponse.success();
    }

    /**
     * 查询会员卡列表
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<List<AppletCardDTO>> getMemberCardByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();
        ServiceResponse<List<MemberCardRelDTO>> appletCardResult = appletsConfigApiService
                .queryAppletIntegralBalanceCardRel(appletConfigId, enterpriseId);
        if (appletCardResult.isSuccess()) {
            List<MemberCardRelDTO> cardList = appletCardResult.getResult();
            List<AppletCardDTO> resultList = new ArrayList<>();
            List<AppletCardDTO> subList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(cardList)) {
                for (MemberCardRelDTO card : cardList) {
                    List<MemberCardRelDTO> sub = card.getRelCards();
                    if (CollectionUtils.isNotEmpty(sub)) {
                        for (MemberCardRelDTO subTemp : sub) {
                            subList.add(new AppletCardDTO().setCardConfigId(subTemp.getId())
                                    .setCardName(subTemp.getName()).setIsSub(true));
                            resultList.add(new AppletCardDTO().setCardConfigId(subTemp.getId())
                                    .setCardName(subTemp.getName()).setIsSub(false));
                        }
                    }
                    resultList.add(new AppletCardDTO().setCardName(card.getName())
                            .setCardConfigId(card.getId()).setSubList(subList).setIsSub(false));
                }
            }
            return ServiceResponse.success();
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), appletCardResult.getMessage());
    }

    /**
     * 获取和主卡关联的店铺
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<ShopDTO> getMainCardStore(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();
        //shopType 1:微商城 2：积分
        ServiceResponse<ShopDTO> shopResult = shopApiService.getShopByAppletId(appletConfigId, enterpriseId, 1);
        if (shopResult.isSuccess()) {
            return ServiceResponse.success(shopResult.getResult());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), shopResult.getMessage());
    }

    /**
     * 根据appId查询小程序配置ID
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<Long> getAppletConfigIdByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<AppletsConfigSimpleOutputDTO> appletResult = appletsConfigApiService
                .selectAppletsConfigIdByAppId(appId, enterpriseId);
        if (appletResult.isSuccess()) {
            return ServiceResponse.success(appletResult.getResult().getAppletConfigId());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), appletResult.getMessage());
    }

}
